/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: basereader.hxx,v $
 * $Revision: 1.5 $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
 
#ifndef BASEREADER_HXX_INCLUDED
#define BASEREADER_HXX_INCLUDED

#include "internal/global.hxx"
#include "internal/types.hxx"
#include "internal/config.hxx"
#include "internal/utilities.hxx"
#include "internal/i_xml_parser_event_handler.hxx"

#ifndef XML_PARSER_HXX_INCLUDED
#include "internal/xml_parser.hxx"
#endif
#include "internal/zipfile.hxx"

class CBaseReader : public i_xml_parser_event_handler
{
public:
	virtual ~CBaseReader();

protected: // protected because its only an implementation relevant class
	CBaseReader( const std::string& DocumentName );

	CBaseReader( void* stream, zlib_filefunc_def* fa );

	virtual void start_document();
	
	virtual void end_document();

	virtual void start_element(
		const std::wstring& raw_name, 
		const std::wstring& local_name, 
		const XmlTagAttributes_t& attributes) = 0;

	virtual void end_element(
		const std::wstring& raw_name, const std::wstring& local_name) = 0;

	virtual void characters(const std::wstring& character) = 0;

	virtual void ignore_whitespace(const std::wstring& /*whitespaces*/){};

	virtual void processing_instruction(
		const std::wstring& /*target*/, const std::wstring& /*data*/){};

	virtual void comment(const std::wstring& /*comment*/){};

	void Initialize( const std::string& /*ContentName*/);

private:
	ZipFile        m_ZipFile;
	ZipFile::ZipContentBuffer_t m_ZipContent;
}; 

#endif
